/**
 * Program skeleton for the course "Programming embedded systems"
 */

#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "stm32f10x_conf.h"
#include "task.h"
#include "setup.h"
#include "assert.h"
#include "semphr.h"

#include "stm32f10x_lib.h"
#include "stm32f10x_map.h"
#include "stm32f10x_wwdg.h"

/*-----------------------------------------------------------*/

xSemaphoreHandle sem;
int WWDGCounter;

/**
 * Immediate interrupt handler
 */
void WWDGHandler(void) {
  portBASE_TYPE higherPrio = pdFALSE;

  /**
   * At this point, it should be checked whether the system is still
   * in a sane state
   */

  WWDGCounter = WWDG->CR;

  // refresh the watchdog timer
  WWDG_SetCounter(0x7F);
  WWDG_ClearFlag();

  xSemaphoreGiveFromISR(sem, &higherPrio);

  // FreeRTOS macro to signal the end of ISR
  portEND_SWITCHING_ISR(higherPrio);
}

#define WINDOW (0x60)

void WWDGPrinter(void *params) {
  for (;;) {
    xSemaphoreTake(sem, portMAX_DELAY);

	printf("refreshing WWDG %d\n", WWDGCounter);
  }
}

/*
 * Entry point of program execution
 */
int main( void )
{
  NVIC_InitTypeDef NVIC_InitStructure;
  prvSetupHardware();

  RCC_APB1PeriphClockCmd(RCC_APB1Periph_WWDG, ENABLE);

  vSemaphoreCreateBinary(sem);
  xSemaphoreTake(sem, portMAX_DELAY);

  xTaskCreate(WWDGPrinter, "printer", 100, NULL, 1, NULL);

  WWDG_DeInit();

  WWDG_SetPrescaler(WWDG_Prescaler_8);
  WWDG_EnableIT();
  WWDG_Enable(0x7F);
  WWDG_SetWindowValue(WINDOW);
  WWDG_ClearFlag();

  /* Configuration of the interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = WWDG_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init( &NVIC_InitStructure );

  vTaskStartScheduler();
  assert(0);
  return 0;                 // not reachable
}
/*-----------------------------------------------------------*/


